--
-- Default vars and utility functions
--
local filters = {}
local defaultFilterList = [[buying.+run[s%s%p]
selling.+run[s%s%p]
wt[bs].+run[s%s%p]
buying.+sto[nc]ks
selling.+sto[nc]ks
wt[bs].+sto[nc]ks
sto[nc]ks? spam
sto[nc]ks? run
buying.+carry
selling.+carry
wt[bs].+carry
selling.+mobs
wts.+mobs
1[%s-]*pull
one[%s-]*pull
mara.+pull
lf.+carry
carries
boo+st
wts.+gold%A
sell.+gold%A
#recruiting
#join.+guild
#wtb.+buff
#wtb.+port
#lf .+port
#buying.+port
#wtb.+summon
#buying.+summon]]
local colorHL = "|cffff55bb"
local colorReg = "|r"

local function trim(str)
    return str:gsub("^%s*(.-)%s*$", "%1")
end

local function lowerPattern(pattern)
    return pattern:gsub("(%%?)(.)", function(percent, char)
        if percent ~= "" then
            return percent .. char
        else
            return char:lower()
        end
    end)
end

local function loadFilters()
    filters = {}
    local i = 1
    for filter in STFU_Config.filters:gmatch("[^\r\n]+") do
        filter = trim(filter)
        -- Lines beginning with a # are comments
        if filter:sub(1, 1) ~= "#" then
            filters[i] = lowerPattern(filter)
            i = i + 1
        end
    end
end

--
-- Set up configuration options panel
--
local panel = CreateFrame("Frame", "STFU_ConfigPanel")
local title = panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
title:SetPoint("TOPLEFT", panel, "TOPLEFT", 11, -15)
title:SetText("Spam Text Filter Ultra " .. GetAddOnMetadata("STFU", "Version"))

local subtitle = panel:CreateFontString(nil, "ARTWORK", "GameFontNormal")
subtitle:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 5, -10)
subtitle:SetTextColor(255/255, 255/255, 255/255)
subtitle:SetText("Enter keywords, phrases, or Lua patterns to filter public chat channels.")

local tips = panel:CreateFontString(nil, "ARTWORK", "GameFontNormal")
tips:SetPoint("TOPLEFT", subtitle, "BOTTOMLEFT", 0, -3)
tips:SetTextColor(180/255, 180/255, 100/255)
tips:SetJustifyH("LEFT")
tips:SetText(" • One pattern per line\n • Patterns are not case sensitive\n • Lines beginning with # are disabled")

local boxWidth = 500
local boxHeight = 350
local scrollWrapper = CreateFrame("Frame", nil, panel, BackdropTemplateMixin and "BackdropTemplate")
scrollWrapper:SetPoint("TOPLEFT", tips, "BOTTOMLEFT", 0, -10)
scrollWrapper:SetSize(boxWidth, boxHeight)
scrollWrapper:SetBackdrop({
    bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
    edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
    edgeSize = 16,
    insets = { left = 3, right = 3, top = 3, bottom = 3 }
})

local scroll = CreateFrame("ScrollFrame", "STFU_ScrollFrame", panel, "UIPanelScrollFrameTemplate")
scroll:SetPoint("TOPLEFT", scrollWrapper, "TOPLEFT", 12, -8)
scroll:SetSize(boxWidth - 16, boxHeight - 16)

local filterBox = CreateFrame("EditBox", nil, panel)
filterBox:SetPoint("TOPLEFT", scroll)
filterBox:SetPoint("BOTTOMRIGHT", scroll)
filterBox:SetSize(boxWidth - 24, boxHeight - 16)
filterBox:SetAutoFocus(false)
filterBox:SetMultiLine(true)
filterBox:SetFontObject("GameFontNormal")
filterBox:SetTextColor(200/255, 200/255, 200/255)
scroll:SetScrollChild(filterBox)

local throttleToggle = CreateFrame("CheckButton", "STFU_ThrottleToggle", panel, "ChatConfigCheckButtonTemplate")
throttleToggle:SetPoint("TOPLEFT", scrollWrapper, "BOTTOMLEFT", 0, -5)
STFU_ThrottleToggleText:SetText(" Throttle messages repeated in less than X seconds: ")

local throttleDisplay = panel:CreateFontString(nil, "ARTWORK", "GameFontNormal")
throttleDisplay:SetTextColor(255/255, 255/255, 255/255)

local throttleDecrease = CreateFrame("Button", "STFU_ThrottleDecrease", panel, "UIPanelButtonTemplate")
throttleDecrease:SetPoint("TOPLEFT", throttleToggle, "TOPRIGHT", 310, -1)
throttleDecrease:SetSize(30, 20)
throttleDecrease:SetText("–")
throttleDecrease:SetScript("OnClick", function()
    local seconds = tonumber(throttleDisplay:GetText())
    if seconds <= 1 then
        return
    end
    throttleDisplay:SetText(seconds - 1)
end)

throttleDisplay:SetPoint("TOPLEFT", throttleDecrease, "TOPRIGHT", 5, -4)

local throttleIncrease = CreateFrame("Button", "STFU_ThrottleDecrease", panel, "UIPanelButtonTemplate")
throttleIncrease:SetPoint("TOPLEFT", throttleDecrease, "TOPRIGHT", 26, 0)
throttleIncrease:SetSize(30, 20)
throttleIncrease:SetText("+")
throttleIncrease:SetScript("OnClick", function()
    local seconds = tonumber(throttleDisplay:GetText())
    if seconds >= 99 then
        return
    end
    throttleDisplay:SetText(seconds + 1)
end)

local debugToggle = CreateFrame("CheckButton", "STFU_DebugToggle", panel, "ChatConfigCheckButtonTemplate")
debugToggle:SetPoint("TOPLEFT", throttleToggle, "BOTTOMLEFT", 0, -5)
STFU_DebugToggleText:SetText(" Debug mode (print filtered messages and invalid patterns in chat)")

panel.name = "STFU"
panel.okay = function()
    STFU_Config.filters = filterBox:GetText()
    STFU_Config.debugMode = debugToggle:GetChecked()
    STFU_Config.throttle = throttleToggle:GetChecked()
    STFU_Config.throttleSeconds = tonumber(throttleDisplay:GetText())
    loadFilters()
end
panel.cancel = function()
    filterBox:SetText(STFU_Config.filters)
    debugToggle:SetChecked(STFU_Config.debugMode)
    throttleToggle:SetChecked(STFU_Config.throttle)
    throttleDisplay:SetText(STFU_Config.throttleSeconds)
end
panel.default = function()
    STFU_Config.filters = defaultFilterList
    filterBox:SetText(defaultFilterList)
    STFU_Config.debugMode = false
    debugToggle:SetChecked(STFU_Config.debugMode)
    STFU_Config.throttle = true
    throttleToggle:SetChecked(STFU_Config.throttle)
    STFU_Config.throttleSeconds = 30
    throttleDisplay:SetText(STFU_Config.throttleSeconds)
    loadFilters()
end
InterfaceOptions_AddCategory(panel)

--
-- Add a convenience slash command for opening the config panel
--
SLASH_STFU1 = "/stfu"
SlashCmdList.STFU = function(msg, editBox)
    InterfaceOptionsFrame_Show()
    InterfaceOptionsFrame_OpenToCategory("STFU")
end

--
-- Load saved filters
--
panel:RegisterEvent("ADDON_LOADED")
panel:RegisterEvent("CHAT_MSG_CHANNEL")
panel:SetScript("OnEvent", function(_, event, addon)
    if event == "ADDON_LOADED" and addon == "STFU" then
        -- Initialize default settings if this is the addon's first load
        if STFU_Config == nil then
            STFU_Config = {}
            STFU_Config.filters = defaultFilterList
            STFU_Config.debugMode = false
            print(colorHL .. "STFU: Thanks for choosing Spam Text Filter Ultra! You can add and remove chat filters by typing /stfu" .. colorReg)
        end
        if STFU_Config.throttle == nil then
            STFU_Config.throttle = true
            STFU_Config.throttleSeconds = 30
        end
        filterBox:SetText(STFU_Config.filters)
        debugToggle:SetChecked(STFU_Config.debugMode)
        throttleToggle:SetChecked(STFU_Config.throttle)
        throttleDisplay:SetText(STFU_Config.throttleSeconds)
        loadFilters()
    end
end)

--
-- Bind chat filter function
--
local lastLineId = 0
local lastResult = false
local prevMsgs = {}

local function testMsgWithFilter(msg, filter)
    return msg:find(filter)
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", function(_, event, msg, author, _, _, _, _, _, _, _, _, lineId, senderGuid)
    -- This event is triggered multiple times for a single chat msg, one for each place the message is displayed
    -- Use the first filter result for all subsequent events
    if lineId == lastLineId then
        return lastResult
    end

    -- Always allow messages from the player
    local player, server = UnitFullName("player")
    if author == player .. "-" .. server then
        lastLineId = lineId
        lastResult = false
        return false
    end

    -- Apply all filters patterns to the chat msg
    msg = msg:lower()
    for i, filter in ipairs(filters) do
        local status, result = pcall(testMsgWithFilter, msg, filter)
        if STFU_Config.debugMode then
            if status == false then
                print(colorHL .. "INVALID PATTERN (line " .. (i + 1) .. "): " .. colorReg .. filter)
            elseif result then
                print(colorHL .. "FILTERED: " .. colorReg .. filter .. colorHL .. " => " .. colorReg .. msg:gsub("item:.-%[", "["))
            end
        end
        if status and result then
            lastLineId = lineId
            lastResult = true
            return true
        end
    end
    
    -- Throttle the chat msg if it has been repeated too quickly by the sender
    if STFU_Config.throttle then
        local now = time()
        local nextIndex = 1
        for i, prevMsg in ipairs(prevMsgs) do
            prevMsgs[i] = nil
            if now - prevMsg.time < STFU_Config.throttleSeconds then
                prevMsgs[nextIndex] = prevMsg
                nextIndex = nextIndex + 1
                if prevMsg.sender == senderGuid and prevMsg.msg == msg then
                    if STFU_Config.debugMode then
                        print(colorHL .. "THROTTLED: " .. colorReg .. author .. colorHL .. " => " .. colorReg .. msg:gsub("item:.-%[", "["))
                    end
                    lastLineId = lineId
                    lastResult = true
                    return true
                end
            end
        end
        prevMsgs[nextIndex] = { sender = senderGuid, msg = msg, time = now }
    end

    lastLineId = lineId
    lastResult = false
    return false
end)